<?php
/**
 * This file is part of SplashSync Project.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 *  @author    Splash Sync <www.splashsync.com>
 *  @copyright 2015-2017 Splash Sync
 *  @license   GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
 *
 **/

namespace   Splash\Local\Objects\Invoice;

/**
 * @abstract    Dolibarr Customer Invoice List Functions
 */
trait ObjectsListTrait
{
    /**
     * @abstract    Build Object Listing Base Sql Query
     *
     * @param        string  $filter                   Filters/Search String for Contact List.
     * @param        array   $params                   Search parameters for result List.
     *
     * @return string
     */
    protected function getSqlBaseRequest($filter = null, $params = null)
    {
        //====================================================================//
        // Prepare SQL request for reading in Database
        //====================================================================//
        $sql    = "SELECT ";
        //====================================================================//
        // Select Database fields
        $sql .= " f.rowid as id,";                  // Object Id
        $sql .= " f.facnumber as ref,";             // Dolibarr Reference
        $sql .= " f.ref_ext as ref_ext,";           // External Reference
        $sql .= " f.ref_int as ref_int,";           // Internal Reference
        $sql .= " f.ref_client as ref_client,";     // Customer Reference
        $sql .= " f.total as total_ht,";            // Total net of tax
        $sql .= " f.total_ttc as total_ttc,";       // Total with tax
        $sql .= " f.datef as date";                 // Invoice date
        //====================================================================//
        // Select Database tables
        $sql   .= " FROM " . MAIN_DB_PREFIX . "facture as f ";
        
        //====================================================================//
        // Entity Filter
        $sql   .= " WHERE f.entity IN (".getEntity('facture', 1).")";
        
        //====================================================================//
        // Setup filters
        //====================================================================//
        // Add filters with names convertions. Added LOWER function to be NON case sensitive
        if (!empty($filter) && is_string($filter)) {
            $sql   .= " AND ( ";
            //====================================================================//
            // Search in Invoice Ref.
            $sql   .= " LOWER( f.facnumber ) LIKE LOWER( '%" . $filter ."%') ";
            //====================================================================//
            // Search in Invoice Internal Ref
            $sql   .= " OR LOWER( f.ref_int ) LIKE LOWER( '%" . $filter ."%') ";
            //====================================================================//
            // Search in Invoice External Ref
            $sql   .= " OR LOWER( f.ref_ext ) LIKE LOWER( '%" . $filter ."%') ";
            //====================================================================//
            // Search in Invoice Customer Ref
            $sql   .= " OR LOWER( f.ref_client ) LIKE LOWER( '%" . $filter ."%') ";
            $sql   .= " ) ";
        }
        //====================================================================//
        // Setup sortorder
        $sortfield = empty($params["sortfield"])?"f.rowid":$params["sortfield"];
        $sortorder = empty($params["sortorder"])?"DESC":$params["sortorder"];
        $sql   .= " ORDER BY " . $sortfield . " " . $sortorder;

        return $sql;
    }
}
